#!/usr/bin/env python

import spark
import ast

"""
    classes ::= rootdef classdefs
    rootdef ::= rootclass { params }
    classdefs ::= classdef
    classdefs ::= classdefs classdef
    classdef ::= ordclass { params }
    params ::= ratedef
    params ::= ceildef
    params ::= descrdef
    params ::= params params
    ratedef ::= rate = number
    ceildef ::= ceil = number
    descrdef ::= descr = string
    rootclass ::= class classid root
    ordclass ::= class classid parent classid
"""

class SimpleParser(spark.GenericParser):
    def __init__(self, start='classes'):
        spark.GenericParser.__init__(self, start)

    def p_classes(self, args):
        """
            classes ::= rootdef classdefs
        """
        return ast.AST(type='classes', kids=args)

    def p_rootdef(self, args):
        """
            rootdef ::= rootclass { params }
        """
        a = ast.AST('root')
        a.classid = args[0][1]
        a.params = args[2]
        return a

    def p_classdefs(self, args):
        """
            classdefs ::= classdef
            classdefs ::= classdefs classdef
        """
        a = ast.AST(type='classdefs', kids=args)
        return a

    def p_classdef(self, args):
        """
            classdef ::= ordclass { params }
        """
        a = ast.AST('classdef')
        a.classid = args[0][1]
        a.parent = args[0][3]
        a.params = args[2]
        return a

    def p_params(self, args):
        """
            params ::= ratedef
            params ::= ceildef
            params ::= descrdef
            params ::= params params
        """
        ret = []
        if len(args) > 1:
            for p in args:
                ret.extend([x for x in p])
            return ret

        return args


    def p_rate(self, args):
        """
            ratedef ::= rate = number
        """
        args[0].value = args[2]
        return args[0]

    def p_ceil(self, args):
        """
            ceildef ::= ceil = number
        """
        args[0].value = args[2]
        return args[0]

    def p_descr(self, args):
        """
            descrdef ::= descr = string
        """
        args[0].value = args[2]
        return args[0]

    def p_rootclass(self, args):
        """
            rootclass ::= class classid root
        """
        return args

    def p_ordclass(self, args):
        """
            ordclass ::= class classid parent classid
        """
        return args
