#!/usr/bin/env python

import spark
import token

class SimpleScanner(spark.GenericScanner):
    def __init__(self):
        spark.GenericScanner.__init__(self)
        self.keywords = [
            'class',
            'rate',
            'ceil',
            'descr',
            'root',
            'parent',
        ]

    def tokenize(self, input):
        self.rv = []
        spark.GenericScanner.tokenize(self, input)
        return self.rv

    def t_whitespace(self, s):
        r'\s+'
        pass

    def t_comment(self, s):
        r'\#.*'
        pass

    def t_semicol(self, s):
        r';'
        self.rv.append(token.Token(type=s, attr=s))

    def t_openblock(self, s):
        r'{'
        self.rv.append(token.Token(type=s, attr=s))
        
    def t_closeblock(self, s):
        r'}'
        self.rv.append(token.Token(type=s, attr=s))

    def t_equal(self, s):
        r'='
        self.rv.append(token.Token(type=s, attr=s))

    def t_number(self, s):
        r'[0-9]+'
        self.rv.append(token.Token(type='number', attr=s))

    def t_classid(self,s):
        r'[0-9]+:[0-9]+'
        self.rv.append(token.Token(type='classid', attr=s))

    def t_keyword(self, s):
        # r' class | irate | iceil | descr '
        self.rv.append(token.Token(type=s, attr=s))

    def t_string(self, s):
        r'[a-zA-Z_]+[0-9]*'
        if s in self.keywords:
            self.t_keyword(s)
        else:
            self.rv.append(token.Token(type='string', attr=s))
